<?php

namespace Drupal\monarch_utilities\Element;

use Drupal\Core\Render\Element\RenderElement;
use Drupal\Core\Render\Element;

/**
 * Provides a lightweight table container render element for use with
 * forms in a table.
 *
 * @see \Drupal\Core\Render\Element\Table for usage details.
 * @see https://www.drupal.org/project/drupal/issues/3016258
 *
 * @RenderElement("table_container")
 */
class TableContainer extends RenderElement {

  /**
   * {@inheritdoc}
   */
  public function getInfo() {
    $class = get_class($this);
    return [
      '#header' => [],
      '#rows' => [],
      '#empty' => '',
      // Render properties.
      '#pre_render' => [
        [$class, 'preRenderTable'],
      ],
      '#theme' => 'table',
    ];
  }

  /**
   * @see \Drupal\Core\Render\Element\Table::preRenderTable()
   */
  public static function preRenderTable($element) {
    foreach (Element::children($element) as $first) {
      $row = ['data' => []];
      // Apply attributes of first-level elements as table row attributes.
      if (isset($element[$first]['#attributes'])) {
        $row += $element[$first]['#attributes'];
      }
      // Turn second-level elements into table row columns.
      // @todo Do not render a cell for children of #type 'value'.
      // @see https://www.drupal.org/node/1248940
      foreach (Element::children($element[$first]) as $second) {
        // Assign the element by reference, so any potential changes to the
        // original element are taken over.
        $column = ['data' => &$element[$first][$second]];

        // Apply wrapper attributes of second-level elements as table cell
        // attributes.
        if (isset($element[$first][$second]['#wrapper_attributes'])) {
          $column += $element[$first][$second]['#wrapper_attributes'];
        }

        $row['data'][] = $column;
      }
      $element['#rows'][] = $row;
    }

    // Take over $element['#id'] as HTML ID attribute, if not already set.
    Element::setAttributes($element, ['id']);

    return $element;
  }

}
